from datetime import datetime, timedelta
from email.utils import parsedate_tz as parsedate, mktime_tz as mktime
import functools
import operator
import os
import simplejson as json
from StringIO import StringIO
from urllib import urlencode
from urllib2 import urlopen as _urlopen, \
    build_opener as build_urlopener, HTTPRedirectHandler
from urlparse import urlparse, urlunparse
try:
    from urlparse import parse_qs
except ImportError:
    from cgi import parse_qs

from google.appengine.ext import db

from .facebook import GraphAPI, GraphAPIError
    
    
#def member_info():
#    return

__all__ = ['FacebookClientError', 'FacebookClient',
           'GraphAPI', 'GraphAPIError']


class _SetToCallable(object):
    __slots__ = ('cb',)
    def __get__(self, instance, owner):
        return self.cb and self.cb()
    def __set__(self, instance, value):
        self.cb = value


class FacebookClientError(Exception):
    _subclasses = frozenset([GraphAPIError])
    def __init__(self, type, message):
        Exception.__init__(self, message)
        self.type = type
    @classmethod
    def __subclasshook__(cls, subclass):
        #raise Exception
        # TODO: this is not working at all
        if subclass in cls._subclasses: return True
        return NotImplemtented
Error = FacebookClientError
#assert issubclass(GraphAPIError, FacebookClientError)
#assert isinstance(GraphAPIError(), FacebookClientError)

class FacebookClientMeta(type):
    def __new__(cls, name, bases, dct):
        if object is not bases[0]:
            assert set(["APP_ID", "APP_SECRET"]).issubset(dct), \
                "The FacebookClient requires attributes " \
                "APP_ID, APP_SECRET"   
        return type.__new__(cls, name, bases, dct)

    def __init__(self, name, bases, dct):
        from .requesthandler import RequestHandlerMeta
        
        super(FacebookClientMeta, self).__init__(name, bases, dct)
        if object is bases[0]: return
        # TODO: this is resetting vars in subclasses without need!
        self._establish_id_and_secret()
        self._establish_redirect_uri()
        self.auth_url = self._auth_url()
        h_cls = self.request_handler_cls
        self._request_handler_cls = \
            isinstance(h_cls, RequestHandlerMeta) and h_cls or RequestHandlerMeta.get(h_cls)
        
        # Allow empty ID (Temp Solution)
        if not self.APP_ID:
            self._disable_authentication()
            
        #TODO: Support for client flow if there's no APP_SECRET?
    
    def _disable_authentication(self):
        self.access_token = False
        self.auth = self._disabled_auth
        
    def _establish_redirect_uri(self):
        parsed = list(urlparse(self.REDIRECT_URI))
        host = os.environ.get('HTTP_HOST') or os.environ['SERVER_NAME']
        scheme = os.environ.get('HTTP_SCHEME', 'http')
        parsed[0] = parsed[0] or scheme
        parsed[1] = parsed[1] or host

        self.REDIRECT_PATH = parsed[2]
        self.REDIRECT_URI = urlunparse(parsed)
        
        # TODO: improve
        parsed[2] = self.LOGOUT_REDIRECT
        self.LOGOUT_REDIRECT = urlunparse(parsed)
        
    def _establish_id_and_secret(self):
         host = os.environ.get('HTTP_HOST') or os.environ['SERVER_NAME']
         if host in getattr(self, 'EXTRA_DOMAINS', ()):
             self.APP_ID = self.EXTRA_DOMAINS[host]['APP_ID']
             self.APP_SECRET = self.EXTRA_DOMAINS[host]['APP_SECRET']
    
    
class _FacebookAuthSession(db.Model):
    token = db.StringProperty()
    user_id = db.StringProperty()
    raw_user = db.BlobProperty()
    expires = db.DateTimeProperty()
    created = db.DateTimeProperty() # Indicated by the FB server
    @staticmethod
    def delete_session(id):
        return db.delete(db.Key.frompath(model.kind(), id))
    @classmethod
    def put_session(cls, id, token, expires, created):
        # 0 and None mean no expires
        obj = cls(
            key_name=id, token=token, 
            expires=expires and created + timedelta(0, expires) or None, 
            created=created)
        obj.put()
        return obj
    @property
    def user(self):
        ru = self.raw_user
        return ru and json.loads(ru)
    def put_user(self, id, user):
        self.raw_user = json.dumps(user)
        self.user_id = id
        self.put()
_FacebookAuthSession.get_session = _FacebookAuthSession.get_by_key_name


class FacebookClient(object):
    __metaclass__ = FacebookClientMeta
    request_handler_cls = 'Django'
    CODE_PARAM = 'code'
    ERROR_PARAM = 'error'
    ERROR_DESC_PARAM = 'error_description'
    SESSION_COOKIE = 'facebook_auth_session_id'
    AUTH_ENDPOINT = 'https://www.facebook.com/dialog/oauth'
    TOKEN_ENDPOINT = 'https://graph.facebook.com/oauth/access_token'
    AUTH_SCOPE = ()
    REDIRECT_URI = "/"
    LOGOUT_REDIRECT = "/logged_out"
    #TODO: 
    # - AUTH_FLOW
    # - ADMINS
    
    _session_cls = _FacebookAuthSession
    
    def __init__(self, request=None, session_id=None, access_token=None, _request_handler=None):
        self.session_id = None
        self._request_handler = _request_handler
        self._access_token = access_token
        self.expires = None
        self._session = None
        if request: 
            self._set_request(request)
        if not self.session_id: 
            self.session_id = session_id 
        self.graph = GraphAPI(lambda: self.access_token)
        #self.lazy = GraphAPI()
        #self.lazy.request = self.lazy.create_request
        #TODO: Add these
        # - self.lazy
        # - self.graph_video
        # - self.fql
   
    @property
    def access_token(self):
        # Always tries to get the access_token from 
        # the Datastore unless it has gotten it before.
        at = self._access_token
        if self.session_id and not at:
            session = self._session_cls.get_session(self.session_id)
            if session:
                self._access_token = session.token
                self.expires = session.expires
                self._session = session
                return self._access_token
        return at
    
    @property
    def user(self):
        # User will be cached in self after is_authenticated()
        # becomes True.
        try:
            u = self._user
        except AttributeError:
            u = None
            if self.is_authenticated():
                session = self._session
                if session:
                    u = session.user
                    if not u:
                        u = self.get_user()
                        session.put_user(
                            self.get_id(u), u)
                else:
                    u = self.get_user()
                self._user = u
        return u
        
    @property
    def id(self):
        u = self.user
        return u and self.get_id(u)
        
    def get_user(self):
        # Must be json encodable.
        # Can be overriden by user.
        return self.graph.get_object("me")
    
    get_id = staticmethod(operator.itemgetter("id"))
            
    def _set_request(self, request):
        self._request_handler = self._request_handler_cls(request, _conf=self)
        self.session_id = self._request_handler.session_id()
        code = self._request_handler.code()
        if code:
            self.authenticate(code)
        
    def is_authenticated(self):
        return bool(self.access_token) and (self.expires is None 
            or datetime.utcnow() < self.expires)

    def request_access(self):
        # TODO: shouldn't this and other functions be called from the request handler?
        return self._request_handler.request_access()        
    
    def authenticate(self, code):
        file = self.urlopen(self._token_url(code))
        
        # Determine date
        date = dict(file.info()).get('date')
        date = date and parsedate(date)
        date = (date and datetime.fromtimestamp(mktime(date)) 
                or datetime.utcnow())
            
        raw = file.read()
        if raw.startswith('access_token='):
            data = parse_qs(raw)
        else:
            data = json.loads(raw)
            raise Error(data['error']['type'],
                        data['error']['message'])
        
        at = data['access_token'][0]
        try:
            expires = int(data['expires'][0])
        except (ValueError, KeyError):
            expires = None

        session = self._session_cls.put_session(
            id=self.session_id, 
            token=at,
            expires=expires, #0 is the same as None (no expires)
            created=date)
        self._access_token = at
        self._expires = session.expires
    
    @classmethod
    def logout(cls, request):
        #self.session_cls.delete_session(self._request_handler.session_id())
        #TODO: organize this better 
        fb = cls(request)
        if fb.is_authenticated():
            url = "https://www.facebook.com/logout.php?next=%s&access_token=%s" % (
                fb.LOGOUT_REDIRECT, fb.access_token)
        else:
            url = fb.LOGOUT_REDIRECT
        response = fb._request_handler.redirect(url)
        fb._request_handler.delete_cookie(response, fb.SESSION_COOKIE)
        return response
    
    @classmethod
    def auth(cls, function):
        #TODO: this assumes that Django is being used!!!
        @functools.wraps(function)
        def w(request, *args, **kwargs):
            try:
                fb = cls(request)
            except Error:
                # handle other error?
                if err.type != "access_denied":
                    raise
                return fb.access_denied() #use handle_access_denied_error instead?
                
            if not fb.user:
                return fb.request_access()
            request.facebook = fb
            try:
                return function(request, *args, **kwargs)
            except GraphAPIError, err: #what about FBError??
                # HARDCODED!!
                if err.type != "OAuthException":
                    raise
                return fb.request_access() #use handle_OAuthException_error instead?
        return w
    
    def access_denied(self):
        return self._request_handler.access_denied()
        
    #TODO: logged out view default method?
    #TODO: error view?
    
    @classmethod
    def _disabled_auth(cls, function):
        @functools.wraps(function)
        def w(request, *args, **kwargs):
            return cls._request_handler_cls._no_app_id()
        return w
            
    @classmethod
    def _auth_url(cls):
        # TODO: take AUTH_FLOW into account 
        params = {'client_id': cls.APP_ID,
                  'redirect_uri': cls.REDIRECT_URI}
                  
        if cls.AUTH_SCOPE:
            params['scope'] = ','.join(cls.AUTH_SCOPE)
        
        return ('%s?%s' % (
                    cls.AUTH_ENDPOINT,
                    urlencode(params)))

    @classmethod
    def _token_url(cls, code):
        return ('%s?%s' % (
                    cls.TOKEN_ENDPOINT,
                    urlencode({
                        'client_id': cls.APP_ID,
                        'redirect_uri': cls.REDIRECT_URI,
                        'client_secret': cls.APP_SECRET,
                        'code': code})))
                        
    urlopen = staticmethod(_urlopen)
    
    
def _location_or_content_to_json(file):
    dct = dict(file.info())
    url = file.geturl()
    location = dict(file.info()).get('location')
    if location:
        return {'location': location} 
    return json.load(file)


#@build_urlopener
class _Ignore302(HTTPRedirectHandler):
    def http_error_302(self, req, fp, code, msg, hdrs):
        r = StringIO()
        r.info = lambda: hdrs
        url = req.get_full_url()
        r.geturl = lambda: url
        return r
_Ignore302 = build_urlopener(_Ignore302)

#TODO: do this inside FacebookClientMeta!!
GraphAPI.install_hook('get_connections', 'picture',
                      json_load=_location_or_content_to_json,
                      urlopener=_Ignore302)    
GraphAPI.access_token = _SetToCallable()

FBClient = FacebookClient