import logging
import random

from .. import Error


class RequestHandlerMeta(type):
    _all = {}
    _builtin = frozenset(['Django'])
    _mod_names = {'Django': 'django_'}
    def __init__(self, name, bases, dct):
        super(RequestHandlerMeta, self).__init__(name, bases, dct)
        if object is bases[0]: return
        self._all[name] = self
    @classmethod
    def get(cls, name):
        h = cls._all.get(name)
        if not h and name in cls._builtin:
            mname = cls._mod_names.get(name) or name.lower()
            hm = __import__(mname, globals(), locals(), [name], 1)
            h = getattr(hm, name)
            logging.debug("Loaded the %s request handler", name)
        return h


class RequestHandler(object):
    __metaclass__ = RequestHandlerMeta
    def __init__(self, request, _conf=None):
        self.c = _conf
        self.request = request

    @staticmethod
    def generate_session_id():
        return random.randrange(0xffffffff)
    
    def session_id(self):
        return self.get_cookies().get(self.c.SESSION_COOKIE)
            
    def request_access(self):
        response = self.redirect(self.c.auth_url)
        self.set_cookie(response, self.c.SESSION_COOKIE, 
                        self.generate_session_id())
        return response
        
    def access_denied(self):
        return self.respond("You have denied us access.")
        
    def logout(self): # response=None):
        #TODO: organize this better
        return
        
    def code(self):
        if self.get_path() != self.c.REDIRECT_PATH:
            return
        params = self.get_params()
        code = params.get(self.c.CODE_PARAM)
        if not code:
            error = params.get(self.c.ERROR_PARAM)
            if error:
                raise Error(error,
                            params.get(self.c.ERROR_DESC_PARAM))
        return code
    
    @classmethod
    def _no_app_id(cls):
        # TODO: organize this better
        return cls.respond("""<pre>
Welcome to your new web site!

You're just a few steps away from integrating it with Facebook and being able to post
messages to Facebook and pull data and photos from Facebook.

Please view the README.txt file in this project on the CoderBuddy web site for 
quick instructions in how to set this up. You can also view support and email links 
on the CoderBuddy site, for further help. You're almost there, just a few quick steps
left!

Thanks!

- Adrian, Luis and the rest of the CoderBuddy team
</pre>""")