"""
    guestbookapp.views
    ===================

"""
from pprint import pformat

from django.views.generic.simple import direct_to_template
from django.shortcuts import redirect
from django import http

from guestbookapp.models import Message

from fbclient import *


class GuestbookFB(FacebookClient):
    # These two values are provided by Facebook after you
    # register your application.
    APP_ID = "434032866613682"
    APP_SECRET = "30c27883333ae42bcba7c160be729f2b"
    
    # Permissions required to access particular data about
    # the user. For example, "user_location" will grant you
    # access to the location of the current user'; 
    # "publish_stream" will allow the app to post to the feed.
    AUTH_SCOPE = (
        "user_location",
        "publish_stream",
    )
    
    # If you need to run this app in other domains (e.g. for
    # testing), reregister your app using the new domain in
    # Facebook, and include the domain here along with the 
    # CLIENT_ID and CLIENT_SECRET.
    EXTRA_DOMAINS = {
        "sherif-fb-up.appspot.com": {
            "APP_ID": "434032866613682",
            "APP_SECRET": "30c27883333ae42bcba7c160be729f2b"
        }
    }
    
    # This class provides an easy way to handle the oauth 
    # protocol. Decorate the view using GuestbookFB.auth
    # and access the instance from within the class:
    # >>> fb = request.facebook
    #
    # Sample API calls:
    # - Get an object of a given id
    # >>> fb.graph.get_object(id)
    # - Get the feed of the current member
    # >>> feed = fb.graph.get_connections("me", "feed")
    # - Comment a post
    # >>> post = feed["data"][0]
    # >>> fb.graph.put_object(post["id"], "comments", message="First!")
    #
    # More information can be found in:
    # - The file fbclient/facebook.py in this project
    # - http://developers.facebook.com/docs/reference/api/


@GuestbookFB.auth
def home(request):
    """A simple guestbook"""
    fb = request.facebook
        
    template_values = {}
    
    if request.method == "POST":
        msg = request.POST.get('message')
        if msg:
            # Get the profile picture.
            # More info can be found here under 'Connections':
            # http://developers.facebook.com/docs/reference/api/user/#picture
            location = fb.graph.get_connections(
                fb.id, "picture", type="small")['location']
            
            
            new_msg = Message(text=msg,
                              user_id=fb.id,
                              name=fb.user['name'],
                              link=fb.user['link'],
                              photo_link=location,
                             )
            new_msg.put()            
            return redirect(home)
            
        template_values['error'] = True
        
    query = Message.all().order('-created');
    template_values['message_list'] = query.fetch(limit=25)
    template_values['user'] = fb.user

    return direct_to_template(request, 'guestbookapp/guestbook.html', template_values)
    
    
@GuestbookFB.auth    
def info(request, id=None):
    """Display info about user with the given id"""
    fb = request.facebook
    
    # If there is no id redirect to the info page of current user
    if not id:
        return redirect(info, id=fb.id)
            
    template_values = {}
    
    # Visit http://developers.facebook.com/docs/reference/api/user/
    # to see how to get more info about a User
    try:
        template_values['friends'] = fb.graph.get_connections(id, "friends")["data"]
    except GraphAPIError, err:
        # An exception is raised if current user doesn't have
        # permission to see friends.
        pass

    # Get the User object
    template_values['user'] = fb.graph.get_object(id)
    
    # Get this link of the profile photo
    template_values['photo_src'] = fb.graph.get_connections(id, "picture", type="normal")["location"]
    
    template_values['url'] = 'http://' + request.get_host()
    template_values['is_current_user'] = fb.id == id
    template_values['new_status'] = request.GET.get('new_status')
    
    return direct_to_template(
        request,
        'guestbookapp/info.html',
        template_values)
        
      
@GuestbookFB.auth
def update_status(request):
    """Update current user's status"""
    if request.method != "POST":
        return http.HttpResponseNotAllowed(["POST"])
        
    fb = request.facebook
    
    #We save the status by calling the put_object method See:    
    #http://developers.facebook.com/docs/reference/api/user/#statuses
    fb.graph.put_object(fb.id, "feed", 
                        message=request.POST["status"])
    
    response = redirect(info, id=fb.id)
    response['Location'] += '?new_status=1' # An easy way of indicated that a new status was posted
    return response
    

# Allow user to log out as required by Facebook Policy.
# http://developers.facebook.com/policy/#policies

logout = GuestbookFB.logout

def logged_out(request):
    
    return direct_to_template(
        request,
        'guestbookapp/logged_out.html',
        {"logged_out": True})